Exception: Tapyrus::PSTT::Error
- Inherits:
-
StandardError
- Object
- StandardError
- Tapyrus::PSTT::Error
- Defined in:
- lib/tapyrus/pstt.rb
Overview
Raised when a PSTT is malformed, or when an operation violates a rule of TIP-0174.
Raised when a PSTT is malformed, or when an operation violates a rule of TIP-0174.