Module: Tapyrus::PSTT::InputTypes

Defined in:
lib/tapyrus/pstt.rb

Overview

Input map key types.

Constant Summary collapse

UTXO =
0x00
PARTIAL_SIG =
0x02
SIGHASH_TYPE =
0x03
REDEEM_SCRIPT =
0x04
BIP32_DERIVATION =
0x06
FINAL_SCRIPTSIG =
0x07
RIPEMD160 =
0x0a
SHA256 =
0x0b
HASH160 =
0x0c
HASH256 =
0x0d
PREVIOUS_TXID =
0x0e
OUTPUT_INDEX =
0x0f
SEQUENCE =
0x10
REQUIRED_TIME_LOCKTIME =
0x11
REQUIRED_HEIGHT_LOCKTIME =
0x12
PROPRIETARY =
0xfc
RESERVED =

Segregated Witness(0x01, 0x05, 0x08, 0x09) and Taproot(0x13..0x18, defined by BIP-371) key types. They have no meaning in Tapyrus. Key types which BIPs later assigned to MuSig2 and other schemes are not listed here, so that they are carried through as unknown records rather than rejected.

[0x01, 0x05, 0x08, 0x09, *(0x13..0x18)].freeze